@@ -10,31 +10,39 @@ use syn::{
     LitStr, Pat, PatType, PathArguments, PathSegment, Type, TypePath,
 };
 
-use crate::path::PathOperation;
+use crate::{ext::ArgValue, path::PathOperation};
 
 use super::{
     Argument, ArgumentIn, ArgumentResolver, PathOperationResolver, PathOperations, PathResolver,
-    ResolvedOperation, ResolvedPath,
+    ResolvedArg, ResolvedOperation, ResolvedPath,
 };
 
 impl ArgumentResolver for PathOperations {
-    fn resolve_path_arguments<'a>(
-        fn_args: &'a Punctuated<FnArg, Comma>,
-        resolved_path: &'a Option<ResolvedPath>,
-    ) -> Option<Vec<Argument<'a>>> {
+    fn resolve_path_arguments(
+        fn_args: &Punctuated<FnArg, Comma>,
+        resolved_path: Option<Vec<ResolvedArg>>,
+    ) -> Option<Vec<Argument<'_>>> {
         resolved_path
-            .as_ref()
             .zip(Self::find_path_pat_type_and_segment(fn_args))
-            .map(|(path, (_, path_segment))| {
+            .map(|(resolved_args, (_, path_segment))| {
                 let types = Self::get_argument_types(path_segment);
 
-                path.args
-                    .iter()
-                    .zip(types.into_iter())
-                    .map(|(name, ty)| Argument {
-                        argument_in: ArgumentIn::Path,
-                        ident: ty,
-                        name: Some(name),
+                resolved_args
+                    .into_iter()
+                    .zip(types)
+                    .map(|(resolved_arg, ty)| {
+                        let name = match resolved_arg {
+                            ResolvedArg::Path(path) => path.name,
+                            ResolvedArg::Query(query) => query.name,
+                        };
+
+                        Argument {
+                            argument_in: ArgumentIn::Path,
+                            ident: Some(ty),
+                            name: Some(Cow::Owned(name)),
+                            is_array: false,
+                            is_option: false,
+                        }
                     })
                     .collect::<Vec<_>>()
             })
@@ -49,35 +57,7 @@ impl PathOperations {
         }
     }
 
-    fn get_argument_names(pat_type: &PatType) -> Vec<&Ident> {
-        match pat_type.pat.as_ref() {
-            Pat::Ident(pat_ident) => {
-                vec![&pat_ident.ident]
-            }
-            Pat::TupleStruct(tuple) => tuple
-                .pat
-                .elems
-                .iter()
-                .flat_map(|pat| match pat {
-                    Pat::Ident(pat_ident) => vec![&pat_ident.ident],
-                    Pat::Tuple(tuple) => tuple
-                        .elems
-                        .iter()
-                        .map(|pat| match pat {
-                            Pat::Ident(pat_ident) => &pat_ident.ident,
-                            _ => abort_call_site!(
-                                "unexpected pat ident in Pat::Tuple expected Pat::Ident"
-                            ),
-                        })
-                        .collect(),
-                    _ => abort_call_site!("unexpected pat type expected Pat::Ident"),
-                })
-                .collect::<Vec<_>>(),
-            _ => abort_call_site!("unexpected pat type expected Pat::Ident or Pat::Tuple"),
-        }
-    }
-
-    fn get_argument_types(path_segment: &PathSegment) -> Vec<&Ident> {
+    fn get_argument_types(path_segment: &PathSegment) -> impl Iterator<Item = &Ident> {
         match &path_segment.arguments {
             PathArguments::AngleBracketed(angle_bracketed) => angle_bracketed
                 .args
@@ -96,8 +76,7 @@ impl PathOperations {
                         )
                     }
                 })
-                .flat_map(|type_path| type_path.path.get_ident())
-                .collect::<Vec<_>>(),
+                .flat_map(|type_path| type_path.path.get_ident()),
             _ => {
                 abort_call_site!("unexpected argument type, expected Path<...> with angle brakets")
             }
@@ -136,7 +115,7 @@ impl PathOperationResolver for PathOperations {
             if is_valid_request_type(attribute.path.get_ident()) {
                 match attribute.parse_args::<Path>() {
                     Ok(path) => Some(ResolvedOperation {
-                        path: path.path,
+                        path: path.0,
                         path_operation: PathOperation::from_ident(
                             attribute.path.get_ident().unwrap(),
                         ),
@@ -154,30 +133,22 @@ impl PathOperationResolver for PathOperations {
     }
 }
 
-struct Path {
-    path: String,
-}
+struct Path(String);
 
 impl Parse for Path {
     fn parse(input: syn::parse::ParseStream) -> syn::Result<Self> {
-        let mut path = String::new();
+        let path = input.parse::<LitStr>()?.value();
 
+        // ignore rest of the tokens from actix-web path attribute macro
         input.step(|cursor| {
             let mut rest = *cursor;
-
-            // parse only first literal item and ignore rest of the tokens from actix_web path attribute macro
             while let Some((tt, next)) = rest.token_tree() {
-                if let proc_macro2::TokenTree::Literal(literal) = tt {
-                    if path.is_empty() {
-                        path.push_str(&literal.to_string().replace('\"', ""));
-                    }
-                }
                 rest = next;
             }
             Ok(((), rest))
         });
 
-        Ok(Self { path })
+        Ok(Self(path))
     }
 }
 
@@ -188,28 +159,37 @@ impl PathResolver for PathOperations {
                 static ref RE: Regex = Regex::new(r"\{[a-zA-Z0-9_][^{}]*}").unwrap();
             }
 
-            let mut args = Vec::<String>::with_capacity(RE.find_iter(path).count());
+            let mut args = Vec::<ResolvedArg>::with_capacity(RE.find_iter(path).count());
             ResolvedPath {
                 path: RE
                     .replace_all(path, |captures: &Captures| {
-                        let mut capture = captures.get(0).unwrap().as_str().to_string();
+                        let mut capture = &captures[0];
+                        let original_name = String::from(capture);
 
                         if capture.contains("_:") {
                             // replace unnamed capture with generic 'arg0' name
-                            args.push(String::from("arg0"));
+                            args.push(ResolvedArg::Path(ArgValue {
+                                name: String::from("arg0"),
+                                original_name,
+                            }));
                             "{arg0}".to_string()
                         } else if let Some(colon) = capture.find(':') {
                             //  replace colon (:) separated regexp with empty string
-                            let end = capture.len() - 1;
-                            capture.replace_range(colon..end, "");
+                            capture = &capture[1..colon];
 
-                            args.push(String::from(&capture[1..capture.len() - 1]));
+                            args.push(ResolvedArg::Path(ArgValue {
+                                name: String::from(capture),
+                                original_name,
+                            }));
 
-                            capture
+                            format!("{{{capture}}}")
                         } else {
-                            args.push(String::from(&capture[1..capture.len() - 1]));
+                            args.push(ResolvedArg::Path(ArgValue {
+                                name: String::from(&capture[1..capture.len() - 1]),
+                                original_name,
+                            }));
                             // otherwise return the capture itself
-                            capture
+                            capture.to_string()
                         }
                     })
                     .to_string(),
@@ -219,6 +199,7 @@ impl PathResolver for PathOperations {
     }
 }
 
+#[inline]
 fn is_valid_request_type(ident: Option<&Ident>) -> bool {
     matches!(ident, Some(operation) if ["get", "post", "put", "delete", "head", "connect", "options", "trace", "patch"]
         .iter().any(|expected_operation| operation == expected_operation))
