@@ -7,7 +7,7 @@
 #![warn(missing_docs)]
 #![warn(rustdoc::broken_intra_doc_links)]
 
-use std::mem;
+use std::{borrow::Cow, mem};
 
 use component::Component;
 use doc_comment::CommentAttributes;
@@ -35,9 +35,9 @@ mod openapi;
 mod path;
 mod security_requirement;
 
-use crate::path::{Path, PathAttr, PathOperation};
+use crate::path::{Path, PathAttr};
 
-#[cfg(feature = "actix_extras")]
+#[cfg(any(feature = "actix_extras", feature = "rocket_extras"))]
 use ext::ArgumentResolver;
 
 #[proc_macro_error]
@@ -386,6 +386,65 @@ pub fn derive_component(input: TokenStream) -> TokenStream {
 /// Leaving empty _`()`_ creates an empty [`SecurityRequirement`][security] this is useful when
 /// security requirement is optional for operation.
 ///
+/// # actix_extras support for actix-web
+///
+/// **actix_extas** feature gives **utoipa** ability to use **actix-web** path operation macros such as `#[get(...)]` to
+/// resolve path for `#[utoipa::path]`. Also it is able to parse the the `path` parameters with types from function arguments
+/// of operation which are defined within type `web::Path<...>`. Allowing you leave out types of parameters in `params(...)`
+/// section of even leave out the section if description is not needed for parameters. Utoipa is only able to resolve
+/// [primitive types][primitive] and [`String`] type. Using other types is undefined behaviour.
+///
+/// See the **actix_extras** in action in examples [todo-actix](https://github.com/juhaku/utoipa/tree/master/examples/todo-actix).
+///
+/// With **actix_extras** feature enabled the you can leave out definitions for **path**, **operation** and **parmater types** [^actix_extras].
+/// ```rust
+/// use actix_web::{get, web, HttpResponse, Responder};
+/// use serde_json::json;
+///
+/// /// Get Pet by id
+/// #[utoipa::path(
+///     responses(
+///         (status = 200, description = "Pet found from database")
+///     ),
+///     params(
+///         ("id", description = "Pet id"),
+///     )
+/// )]
+/// #[get("/pet/{id}")]
+/// async fn get_pet_by_id(id: web::Path<i32>) -> impl Responder {
+///     HttpResponse::Ok().json(json!({ "pet": format!("{:?}", &id.into_inner()) }))
+/// }
+/// ```
+///
+/// With **actix_extras** you may also not to list any _**parmas**_ if you do not want to specify any description for them. Params are resolved from
+/// path and the argument types of handler. [^actix_extras]
+/// ```rust
+/// use actix_web::{get, web, HttpResponse, Responder};
+/// use serde_json::json;
+///
+/// /// Get Pet by id
+/// #[utoipa::path(
+///     responses(
+///         (status = 200, description = "Pet found from database")
+///     )
+/// )]
+/// #[get("/pet/{id}")]
+/// async fn get_pet_by_id(id: web::Path<i32>) -> impl Responder {
+///     HttpResponse::Ok().json(json!({ "pet": format!("{:?}", &id.into_inner()) }))
+/// }
+/// ```
+///
+/// # rocket_extras support for rocket
+///
+/// **rocket_extras** feature give **utoipa** ability to use **rocket** path operation macros such as `#[get(...)]` to
+/// resolve path for `#[utoipa::path]`.  Also it is able to parse the `path` and `query` parameters from path operation macro
+/// combined with function arguments of the operation. Allowing you leave out types from parameters in `params(...)` section
+/// or even leave out the section if description is not needed for parameters. Utoipa is only able to parse parameter types
+/// for [primitive types][primitive], [`String`], [`Vec`], [`Option`] or [`std::path::PathBuf`] type. Other function arguments are
+/// simply ignored.
+///
+/// See the **rocket_extras** in action in examples [rocket-todo](https://github.com/juhaku/utoipa/tree/master/examples/rocket-todo).
+///
 /// # Examples
 ///
 /// Example with all possible arguments.
@@ -456,44 +515,6 @@ pub fn derive_component(input: TokenStream) -> TokenStream {
 /// }
 /// ```
 ///
-/// With **actix_extras** feature enabled the you can leave out definitions for **path**, **operation** and **parmater types** [^actix_extras].
-/// ```rust
-/// use actix_web::{get, web, HttpResponse, Responder};
-/// use serde_json::json;
-///
-/// /// Get Pet by id
-/// #[utoipa::path(
-///     responses(
-///         (status = 200, description = "Pet found from database")
-///     ),
-///     params(
-///         ("id", description = "Pet id"),
-///     )
-/// )]
-/// #[get("/pet/{id}")]
-/// async fn get_pet_by_id(id: web::Path<i32>) -> impl Responder {
-///     HttpResponse::Ok().json(json!({ "pet": format!("{:?}", &id.into_inner()) }))
-/// }
-/// ```
-///
-/// With **actix_extras** you may also not to list any _**parmas**_ if you do not want to specify any description for them. Params are resolved from
-/// path and the argument types of handler.
-/// ```rust
-/// use actix_web::{get, web, HttpResponse, Responder};
-/// use serde_json::json;
-///
-/// /// Get Pet by id
-/// #[utoipa::path(
-///     responses(
-///         (status = 200, description = "Pet found from database")
-///     )
-/// )]
-/// #[get("/pet/{id}")]
-/// async fn get_pet_by_id(id: web::Path<i32>) -> impl Responder {
-///     HttpResponse::Ok().json(json!({ "pet": format!("{:?}", &id.into_inner()) }))
-/// }
-/// ```
-///
 /// Use of Rust's own `#[deprecated]` attribute will refect to the generated OpenAPI spec and mark this operation as deprecated.
 /// ```rust
 /// # use actix_web::{get, web, HttpResponse, Responder};
@@ -540,32 +561,31 @@ pub fn derive_component(input: TokenStream) -> TokenStream {
 pub fn path(attr: TokenStream, item: TokenStream) -> TokenStream {
     let path_attribute = syn::parse_macro_input!(attr as PathAttr);
 
-    #[cfg(feature = "actix_extras")]
+    #[cfg(any(feature = "actix_extras", feature = "rocket_extras"))]
     let mut path_attribute = path_attribute;
 
     let ast_fn = syn::parse::<ItemFn>(item).unwrap_or_abort();
     let fn_name = &*ast_fn.sig.ident.to_string();
 
-    let resolved_operation = &mut PathOperations::resolve_operation(&ast_fn);
-    let resolved_path = PathOperations::resolve_path(
+    let mut resolved_operation = PathOperations::resolve_operation(&ast_fn);
+
+    let mut resolved_path = PathOperations::resolve_path(
         &resolved_operation
             .as_mut()
             .map(|operation| mem::take(&mut operation.path))
             .or_else(|| path_attribute.path.as_ref().map(String::to_string)), // cannot use mem take because we need this later
     );
 
-    #[cfg(feature = "actix_extras")]
+    #[cfg(any(feature = "actix_extras", feature = "rocket_extras"))]
     {
-        let arguments = PathOperations::resolve_path_arguments(&ast_fn.sig.inputs, &resolved_path);
+        let args = resolved_path.as_mut().map(|path| mem::take(&mut path.args));
+        let arguments = PathOperations::resolve_path_arguments(&ast_fn.sig.inputs, args);
+
         path_attribute.update_parameters(arguments);
     }
 
     let path = Path::new(path_attribute, fn_name)
-        .path_operation(
-            resolved_operation.as_mut().map(|operation| {
-                mem::replace(&mut operation.path_operation, PathOperation::Options)
-            }),
-        )
+        .path_operation(resolved_operation.map(|operation| operation.path_operation))
         .path(|| resolved_path.map(|path| path.path))
         .doc_comments(CommentAttributes::from_attributes(&ast_fn.attrs).0)
         .deprecated(ast_fn.attrs.iter().find_map(|attr| {
@@ -777,7 +797,7 @@ impl ToTokens for Required {
     }
 }
 
-/// Parses a type information in uotapi macro parameters.
+/// Parses a type information in utoipa macro parameters.
 ///
 /// Supports formats:
 ///   * `type` type is just a simple type identifier
@@ -785,24 +805,24 @@ impl ToTokens for Required {
 ///   * `Option<type>` type is option of type
 ///   * `Option<[type]>` type is an option of array of types
 #[cfg_attr(feature = "debug", derive(Debug))]
-struct Type {
-    ty: Ident,
+struct Type<'a> {
+    ty: Cow<'a, Ident>,
     is_array: bool,
     is_option: bool,
 }
 
-impl Type {
-    #[cfg(feature = "actix_extras")]
-    pub fn new(ident: Ident) -> Self {
+impl<'a> Type<'a> {
+    #[cfg(any(feature = "actix_extras", feature = "rocket_extras"))]
+    pub fn new(ident: Cow<'a, Ident>, is_array: bool, is_option: bool) -> Self {
         Self {
             ty: ident,
-            is_array: false,
-            is_option: false,
+            is_array,
+            is_option,
         }
     }
 }
 
-impl Parse for Type {
+impl Parse for Type<'_> {
     fn parse(input: ParseStream) -> syn::Result<Self> {
         let mut is_array = false;
         let mut is_option = false;
@@ -838,7 +858,7 @@ impl Parse for Type {
         }?;
 
         Ok(Type {
-            ty,
+            ty: Cow::Owned(ty),
             is_array,
             is_option,
         })
